==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/personalistika/dovednostosobyform.php <==
<?php
/**
 * stanislav.elbl, 24.03.2021
 */

namespace RonPortal\Personalistika;

use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


class OsobaDovednostField extends OsobaFormField
{
    public $idPP;

    public function __construct($prava, $datumPlatnostiPP, $idPP = null)
    {
        parent::__construct($prava, $datumPlatnostiPP, $idPP);
        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this->idPP !== 0)
            $value = $this->idPP;

        //$canEdit = $this->editable;
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

/**
 * Zobrazi pole pro vyber dovednosti (pokud je editovatelne)
 */
class DovednostField extends FormField
{
    private $idPP;

    /**
     *
     * @param string $caption Nadpis pole
     * @param int $idPP id vybraneho pracovniho pomeru
     */
    function __construct($idPP)
    {
        parent::__construct();

        $this->fieldType = 'dovednost';

        $this->idPP = $idPP;

        $this->otherHtmlParams = ['onChange' => 'personalistika.zobrazUrovneDovednosti(this.value);'];  // TODO: prehodit do .js
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     * @throws \Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $edit = (isset($_POST[$this->fieldName . "_edit"]) ? $_POST[$this->fieldName . "_edit"] : $this->editable);

        if ($edit) {
            //if ( !isset($value) ) $value = RonPortalSession::vybranaOsoba();

            $combo = new ComboBox($this->fieldName); // jmeno promenne

            $filtrTPM = '';

            $typRozdeleniTPM = RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');

            //pokud je v nastaveni zvolen typ rozdeleni pro typova pracovni mista, bude mozne zamestnanci zadat jen dovednosti
            // pozadovane pro rozdeleni (typove pracovni misto) zvoleneho zamestnance
            if (($typRozdeleniTPM != -1) && ($typRozdeleniTPM != 0)) {
                $filtrTPM =
                    ' AND EXISTS(SELECT Rozdeleni FROM RozdeleniOsob
                                   JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo
                                        AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                                   JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni
                                        AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni)
                                 WHERE PracovniPomer.ID = ' . (int)$this->idPP .
                    '              AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleniTPM .
                    '              AND RozdeleniOsob.PlatnostOd <= ' . $database->timestampToString(DateUtils::today()) .
                    '              AND (RozdeleniOsob.PlatnostDo >= ' . $database->timestampToString(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.KodDovednosti = Dovednost.Kod ' .
                    ')';
            }

            $sql = "SELECT Kod, Popis FROM Dovednost " .
                "WHERE Aktivni = 1" .
                "  AND EXISTS(SELECT KodUrovne FROM UrovenDovednosti WHERE KodDovednosti = Dovednost.Kod)" .
                //17.3.2017 RON-PR dovednosti, ktere jsou slozene nebo definovane hodnotou nelze rucne zadat
                "  AND Slozena = 0 " .
                "  AND DefinovanaHodnotou = 0
                    $filtrTPM
                    ORDER BY Popis";

            $combo->emptyRow = ' ';
            $combo->loadData($database, $sql, $this->defaultValue);
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->render();
        } else {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->defaultValue . '" />';
            echo '<input type="hidden" name="' . $this->fieldName . "_edit" . '" value="' . ($this->readOnly ? '0' : '1') . '" />';
            echo $this->loadName($database, $this->defaultValue) . "\n";
        }
    }

    function createDBRecordField($database, $tablename, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        $field->value = trim($this ->getPostValueText($utf8));

        return $field;
    }

    /**
     *
     * @param Database $database
     * @param $value
     * @return string
     * @throws \Exception
     */
    function loadName($database, $value)
    {
        $params = $database->createParameters();
        $sql = "SELECT Popis FROM Dovednost WHERE Kod = " . $params->addParameter($value);

        $cursor = $database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            throw new \Exception('Chyba - dovednost: ' . $this->defaultValue . ' nebyla nalezena!' . $sql);
        }

        return $row[0];
    }

}

class DovednostOsobyForm extends RonPortalForm
{
    const XML_EVENT_BEFORE = 'DOVEDNOST_BEFORE';
    const XML_EVENT_AFTER = 'DOVEDNOST_AFTER';
    const XML_CONTEXT_INS = 'insert';
    const XML_CONTEXT_UPD = 'update';

    private $prava;
    private $pracPomer = null;
    private $dovednost;
    private $editDovednost = true;

    /**
     * DovednostForm constructor.
     * @param Database $database
     * @param int $idPP
     * @throws \Exception
     */
    function __construct($database, $idPP = 0)
    {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(),time());
        $this ->prava = Prava::vratPrava('doved_edit');

        if ($idPP) {
            $this ->pracPomer = PracovniPomer::nactiPP($this ->database, $idPP);
        }
        $this->nactiParametr();

        if (isset($_POST['dovednost'])) { $this->dovednost = filter_input(INPUT_POST, 'dovednost'); }

        $this ->htmlAction = '?menuid=maticedovednosti';
        if (Prava::vratHodnotuPrava('doved_ziskane') != 0) {
            $stornoURL = '?menuid=maticedovednosti&action='.$_SESSION['dovednostibrowser'];
        } else {
            $stornoURL = '?menuid=maticedovednosti&action=novadovednost';
        }

        $this -> stornoAction = $stornoURL;

        $this -> sqlTableName = 'DovednostOsoby';

        //$this -> dbLogSrcDescription = 'Formular nove dovednosti osoby';
        $this ->code = 'formdovednostosoby';

        $this->sqlSelectPart =
            'DovednostOsoby.RC, DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, DovednostOsoby.PlatnostDo, '. //0 - 4
            $this->database->castWideMemo('DovednostOsoby.Poznamka') . ', ' . // 5
            'DovednostOsoby.Insert_UserName, DovednostOsoby.Insert_Timestamp, DovednostOsoby.Update_UserName, DovednostOsoby.Update_Timestamp, '. //6 - 9
            'DovednostOsoby.SpecifikaceUrovne'; //10

        $this->addField(new OsobaDovednostField('doved_edit', time(), $this ->pracPomer ? $this ->pracPomer ->idPP : null))
            ->setCaption('Osoba') ->setReadOnly(true)
            ->setCode('field-rc')
            ->setOtherHtmlParams([
                'class' => 'form-control ronselect',
                'data-columns' => 'text;osobni-cislo',
                'data-captions' => TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo'),
                'onChange' => "window.location = &quot;?menuid=maticedovednosti&formaction={$this->formAction}&idosoby=&quot;+this.value;"]); // TODO: prehodit do .js

        $this->addField(new DovednostField($this->pracPomer ? $this->pracPomer->idPP : null))
            ->setFieldName('dovednost') ->setSqlFieldName('DovednostOsoby.KodDovednosti') ->setRequired(true)
            ->setCode('field-dovednost')
            ->setCaption('Dovednosti_dovednost_form') ->setDefaultValue($this->dovednost)
            ->setReadOnly(!$this ->editDovednost);

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('urovenDovednosti') ->setSqlFieldName('DovednostOsoby.KodUrovne') ->setSqlFieldIndex(2) ->setRequired(true)
            ->setCode('field-kodurovne')
            ->setRowId('urovneDovednosti') ->setPrimary(true) ->setCaption('Dovednosti_uroven_dovednosti')
            ->setDataType('int')
            ->setSelectSql(new SqlStatement(
                    "SELECT KodUrovne, Popis FROM UrovenDovednosti" .
                    " WHERE KodDovednosti = " . $params ->addParameter($this->dovednost) .
                    " ORDER BY KodUrovne, Popis",
                    $params)
            );

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('specifikaceUrovneDovednosti') ->setSqlFieldName('DovednostOsoby.SpecifikaceUrovne') ->setSqlFieldIndex(10)
            ->setCode('field-specifikaceurovne')
            ->setCaption('Dovednosti_specifikace_urovne')->setRowId('specifikaceUrovneDovednosti') ->setRequired(true) ->setDefaultValue(0)
            ->setDataType('int') ->setSelectDataTypeTest(false)
            ->setSelectSql(
                "SELECT Kod, Popis FROM S_SpecUrovneDovednosti " .
                //"WHERE KodDovednosti = '" . $database->addSlashes($this->f_dovednost) . "'" .
                "ORDER BY Kod, Popis");

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumZiskani') ->setSqlFieldName('DovednostOsoby.DatumZiskani')
            ->setCode('field-datumziskani')
            ->setSqlFieldIndex(3) ->setCaption('Dovednosti_datum_ziskani') ->setPrimary(true)
            ->setRequired(true) ->setMaxLength(10)
            ->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        //[FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostDo', FORM_FIELD_SQL_NAME => 'DovednostOsoby.PlatnostDo', FORM_FIELD_SQL_INDEX => 4, FORM_FIELD_CAPTION => $STR_PlatnostDo, FORM_FIELD_MAXLENGTH => 10], nastavuje agent

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setSqlFieldName('DovednostOsoby.Poznamka') ->setSqlFieldIndex(5)
            ->setCode('field-poznamka')
            ->setCaption('Poznamka');

//        $this ->addField(FormField::FIELD_TYPE_FILE)
//            ->setCode('field-dokument')
//            ->setFieldName('Dokument');

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Insert_Timestamp') ->setSqlFieldIndex(7) ->setReadOnly(true)
            ->setCode('field-insert_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Insert_UserName') ->setSqlFieldIndex(6) ->setReadOnly(true)
            ->setCode('field-insert_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Update_Timestamp') ->setSqlFieldIndex(9)
            ->setCode('field-update_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Update_UserName') ->setSqlFieldIndex(8)
            ->setCode('field-update_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());
    }

    /**
     *
     * Nacte z URL adresy osobu, (dovednost)
     * @return void Nacte z URL adresy osobu, (dovednost)
     */
    function nactiParametr()
    {
        if (isset($_GET['dovednost'])) {
            $this ->dovednost = $_GET['dovednost'];

            //pokud lze zobrazit matici ziskanych dovednosti, nebude pro prokliku z matice (pri zvolene dovednosti) aktivni vyber dovednosti
            if (Prava::vratHodnotuPrava('doved_ziskane') != 0)
                $this->editDovednost = false;
        }
    }

    function getWhereCondition()
    {
        $params = $this->database->createParameters();
        return new SqlStatement(
            "RC = " . $params->addParameter($this->pracPomer->rc) .
            " AND KodDovednosti = " . $params->addParameter($this->dovednost), $params
        );
    }



    function createDBRecordFields($rec, $insert)
    {
        $rec->addField('RC', '', true, 15, $this->pracPomer->rc);
        parent::createDBRecordFields($rec, $insert);
    }


    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude vkladat
     * @return bool vraci true, pokud muze byt zaznam vlozen
     * @throws \Exception
     */
    function beforeInsertRecord($rec)
    {
        if (!$this->prava->lzeZobrazitRC($rec['RC'])) {
            RonPortalUtils::neniPravoException();
        }

        if ($this->checkUrovenDovednosti($rec['RC'], $rec['KodDovednosti'], $rec['KodUrovne'], $rec['DatumZiskani'])) {
            throw new \Exception(TextManager::_('Dovednosti_uroven_existuje'));
        }

        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude editovat
     * @return boolean vraci true, pokud muze byt zaznam editovan
     */
    function beforeUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl vlozen
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterInsertRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl editovan
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }


    /**
     *
     * @param string $rc
     * @param string $dovednost
     * @param string $uroven
     * @param int $datumZiskani
     * @return bool true pokud uz uroven ma
     * @throws DBException
     */
    function checkUrovenDovednosti($rc, $dovednost, $uroven, $datumZiskani)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT RC FROM DovednostOsoby" .
            " WHERE RC = " . $params->addParameter($rc) . " AND KodDovednosti = " . $params->addParameter($dovednost) .
            " AND KodUrovne = " . $params->addParameter($uroven) .
            " AND DatumZiskani = " . $params->addParameter($this->database->dateToParam($datumZiskani));

        $cursor = $this->database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            return false;
        }

        if ($row[0]) {
            return true;
        }

        return false;
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2305_171.zip: ronportal/ronportal/personalistika/dovednostosobyform.php <==
<?php
/**
 * stanislav.elbl, 24.03.2021
 */

namespace RonPortal\Personalistika;

use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


class OsobaDovednostField extends OsobaFormField
{
    public $idPP;

    public function __construct($prava, $datumPlatnostiPP, $idPP = null)
    {
        parent::__construct($prava, $datumPlatnostiPP, $idPP);
        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this->idPP !== 0)
            $value = $this->idPP;

        //$canEdit = $this->editable;
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

/**
 * Zobrazi pole pro vyber dovednosti (pokud je editovatelne)
 */
class DovednostField extends FormField
{
    private $idPP;

    /**
     *
     * @param string $caption Nadpis pole
     * @param int $idPP id vybraneho pracovniho pomeru
     */
    function __construct($idPP)
    {
        parent::__construct();

        $this->fieldType = 'dovednost';

        $this->idPP = $idPP;

        $this->otherHtmlParams = ['onChange' => 'personalistika.zobrazUrovneDovednosti(this.value);'];  // TODO: prehodit do .js
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     * @throws \Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $edit = (isset($_POST[$this->fieldName . "_edit"]) ? $_POST[$this->fieldName . "_edit"] : !$this->readOnly);

        if ($edit) {
            //if ( !isset($value) ) $value = RonPortalSession::vybranaOsoba();

            $combo = new ComboBox($this->fieldName); // jmeno promenne

            $filtrTPM = '';

            $typRozdeleniTPM = RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');

            //pokud je v nastaveni zvolen typ rozdeleni pro typova pracovni mista, bude mozne zamestnanci zadat jen dovednosti
            // pozadovane pro rozdeleni (typove pracovni misto) zvoleneho zamestnance
            if (($typRozdeleniTPM != -1) && ($typRozdeleniTPM != 0)) {
                $filtrTPM =
                    ' AND EXISTS(SELECT Rozdeleni FROM RozdeleniOsob
                                   JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo
                                        AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                                   JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni
                                        AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni)
                                 WHERE PracovniPomer.ID = ' . (int)$this->idPP .
                    '              AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleniTPM .
                    '              AND RozdeleniOsob.PlatnostOd <= ' . $database->timestampToString(DateUtils::today()) .
                    '              AND (RozdeleniOsob.PlatnostDo >= ' . $database->timestampToString(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.KodDovednosti = Dovednost.Kod ' .
                    ')';
            }

            $sql = "SELECT Kod, Popis FROM Dovednost " .
                "WHERE Aktivni = 1" .
                "  AND EXISTS(SELECT KodUrovne FROM UrovenDovednosti WHERE KodDovednosti = Dovednost.Kod)" .
                //17.3.2017 RON-PR dovednosti, ktere jsou slozene nebo definovane hodnotou nelze rucne zadat
                "  AND Slozena = 0 " .
                "  AND DefinovanaHodnotou = 0
                    $filtrTPM
                    ORDER BY Popis";

            $combo->emptyRow = ' ';
            $combo->loadData($database, $sql, $this->defaultValue);
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->render();
        } else {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->defaultValue . '" />';
            echo '<input type="hidden" name="' . $this->fieldName . "_edit" . '" value="' . ($this->readOnly ? '0' : '1') . '" />';
            echo $this->loadName($database, $this->defaultValue) . "\n";
        }
    }

    function createDBRecordField($database, $tablename, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, '', $this->readOnly, $this->maxLength);
        $field->value = trim($this ->getPostValueText($utf8));

        return $field;
    }

    /**
     *
     * @param Database $database
     * @param $value
     * @return string
     * @throws \Exception
     */
    function loadName($database, $value)
    {
        $params = $database->createParameters();
        $sql = "SELECT Popis FROM Dovednost WHERE Kod = " . $params->addParameter($value);

        $cursor = $database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            throw new \Exception('Chyba - dovednost: ' . $this->defaultValue . ' nebyla nalezena!' . $sql);
        }

        return $row[0];
    }

}

class DovednostOsobyForm extends RonPortalForm
{
    const XML_EVENT_BEFORE = 'DOVEDNOST_BEFORE';
    const XML_EVENT_AFTER = 'DOVEDNOST_AFTER';
    const XML_CONTEXT_INS = 'insert';
    const XML_CONTEXT_UPD = 'update';

    private $prava;
    private $pracPomer = null;
    private $dovednost;
    private $editDovednost = true;

    /**
     * DovednostForm constructor.
     * @param Database $database
     * @param int $idPP
     * @throws \Exception
     */
    function __construct($database, $idPP = 0)
    {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(),time());
        $this ->prava = Prava::vratPrava('doved_edit');

        if ($idPP) {
            $this ->pracPomer = PracovniPomer::nactiPP($this ->database, $idPP);
        }
        $this->nactiParametr();

        if (isset($_POST['dovednost'])) { $this->dovednost = filter_input(INPUT_POST, 'dovednost'); }

        $this ->htmlAction = '?menuid=maticedovednosti';
        if (Prava::vratHodnotuPrava('doved_ziskane') != 0) {
            $stornoURL = '?menuid=maticedovednosti&action='.$_SESSION['dovednostibrowser'];
        } else {
            $stornoURL = '?menuid=maticedovednosti&action=novadovednost';
        }

        $this -> stornoAction = $stornoURL;

        $this -> sqlTableName = 'DovednostOsoby';

        //$this -> dbLogSrcDescription = 'Formular nove dovednosti osoby';
        $this ->code = 'formdovednostosoby';

        $this->sqlSelectPart =
            'DovednostOsoby.RC, DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, DovednostOsoby.PlatnostDo, '. //0 - 4
            $this->database->castWideMemo('DovednostOsoby.Poznamka') . ', ' . // 5
            'DovednostOsoby.Insert_UserName, DovednostOsoby.Insert_Timestamp, DovednostOsoby.Update_UserName, DovednostOsoby.Update_Timestamp, '. //6 - 9
            'DovednostOsoby.SpecifikaceUrovne'; //10

        $this->addField(new OsobaDovednostField('doved_edit', time(), $this ->pracPomer ? $this ->pracPomer ->idPP : null))
            ->setCaption('Osoba') ->setReadOnly(true)
            ->setCode('field-rc')
            ->setOtherHtmlParams([
                'class' => 'form-control ronselect',
                'data-columns' => 'text;osobni-cislo',
                'data-captions' => TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo'),
                'onChange' => "window.location = &quot;?menuid=maticedovednosti&formaction={$this->formAction}&idosoby=&quot;+this.value;"]); // TODO: prehodit do .js

        $this->addField(new DovednostField($this->pracPomer ? $this->pracPomer->idPP : null))
            ->setFieldName('dovednost') ->setSqlFieldName('DovednostOsoby.KodDovednosti') ->setRequired(true)
            ->setCode('field-dovednost')
            ->setCaption('Dovednosti_dovednost_form') ->setDefaultValue($this->dovednost)
            ->setReadOnly(!$this ->editDovednost);

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('urovenDovednosti') ->setSqlFieldName('DovednostOsoby.KodUrovne') ->setSqlFieldIndex(2) ->setRequired(true)
            ->setCode('field-kodurovne')
            ->setRowId('urovneDovednosti') ->setPrimary(true) ->setCaption('Dovednosti_uroven_dovednosti')
            ->setDataType('int')
            ->setSelectSql(new SqlStatement(
                    "SELECT KodUrovne, Popis FROM UrovenDovednosti" .
                    " WHERE KodDovednosti = " . $params ->addParameter($this->dovednost) .
                    " ORDER BY KodUrovne, Popis",
                    $params)
            );

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('specifikaceUrovneDovednosti') ->setSqlFieldName('DovednostOsoby.SpecifikaceUrovne') ->setSqlFieldIndex(10)
            ->setCode('field-specifikaceurovne')
            ->setCaption('Dovednosti_specifikace_urovne')->setRowId('specifikaceUrovneDovednosti') ->setRequired(true) ->setDefaultValue(0)
            ->setDataType('int') ->setSelectDataTypeTest(false)
            ->setSelectSql(
                "SELECT Kod, Popis FROM S_SpecUrovneDovednosti " .
                //"WHERE KodDovednosti = '" . $database->addSlashes($this->f_dovednost) . "'" .
                "ORDER BY Kod, Popis");

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumZiskani') ->setSqlFieldName('DovednostOsoby.DatumZiskani')
            ->setCode('field-datumziskani')
            ->setSqlFieldIndex(3) ->setCaption('Dovednosti_datum_ziskani') ->setPrimary(true)
            ->setRequired(true) ->setMaxLength(10)
            ->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        //[FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostDo', FORM_FIELD_SQL_NAME => 'DovednostOsoby.PlatnostDo', FORM_FIELD_SQL_INDEX => 4, FORM_FIELD_CAPTION => $STR_PlatnostDo, FORM_FIELD_MAXLENGTH => 10], nastavuje agent

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setSqlFieldName('DovednostOsoby.Poznamka') ->setSqlFieldIndex(5)
            ->setCode('field-poznamka')
            ->setCaption('Poznamka');

//        $this ->addField(FormField::FIELD_TYPE_FILE)
//            ->setCode('field-dokument')
//            ->setFieldName('Dokument');

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Insert_Timestamp') ->setSqlFieldIndex(7) ->setReadOnly(true)
            ->setCode('field-insert_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Insert_UserName') ->setSqlFieldIndex(6) ->setReadOnly(true)
            ->setCode('field-insert_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('DovednostOsoby.Update_Timestamp') ->setSqlFieldIndex(9)
            ->setCode('field-update_timestamp');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('Update_UserName') ->setSqlFieldIndex(8)
            ->setCode('field-update_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());
    }

    /**
     *
     * Nacte z URL adresy osobu, (dovednost)
     * @return void Nacte z URL adresy osobu, (dovednost)
     */
    function nactiParametr()
    {
        if (isset($_GET['dovednost'])) {
            $this ->dovednost = $_GET['dovednost'];

            //pokud lze zobrazit matici ziskanych dovednosti, nebude pro prokliku z matice (pri zvolene dovednosti) aktivni vyber dovednosti
            if (Prava::vratHodnotuPrava('doved_ziskane') != 0)
                $this->editDovednost = false;
        }
    }

    function getWhereCondition()
    {
        $params = $this->database->createParameters();
        return new SqlStatement(
            "RC = " . $params->addParameter($this->pracPomer->rc) .
            " AND KodDovednosti = " . $params->addParameter($this->dovednost), $params
        );
    }



    function createDBRecordFields($rec, $insert)
    {
        $rec->addField('RC', '', true, 15, $this->pracPomer->rc);
        parent::createDBRecordFields($rec, $insert);
    }


    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude vkladat
     * @return bool vraci true, pokud muze byt zaznam vlozen
     * @throws \Exception
     */
    function beforeInsertRecord($rec)
    {
        if (!$this->prava->lzeZobrazitRC($rec['RC'])) {
            RonPortalUtils::neniPravoException();
        }

        if ($this->checkUrovenDovednosti($rec['RC'], $rec['KodDovednosti'], $rec['KodUrovne'], $rec['DatumZiskani'])) {
            throw new \Exception(TextManager::_('Dovednosti_uroven_existuje'));
        }

        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude editovat
     * @return boolean vraci true, pokud muze byt zaznam editovan
     */
    function beforeUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl vlozen
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterInsertRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl editovan
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }


    /**
     *
     * @param string $rc
     * @param string $dovednost
     * @param string $uroven
     * @param int $datumZiskani
     * @return bool true pokud uz uroven ma
     * @throws DBException
     */
    function checkUrovenDovednosti($rc, $dovednost, $uroven, $datumZiskani)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT RC FROM DovednostOsoby" .
            " WHERE RC = " . $params->addParameter($rc) . " AND KodDovednosti = " . $params->addParameter($dovednost) .
            " AND KodUrovne = " . $params->addParameter($uroven) .
            " AND DatumZiskani = " . $params->addParameter($this->database->dateToParam($datumZiskani));

        $cursor = $this->database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            return false;
        }

        if ($row[0]) {
            return true;
        }

        return false;
    }
}
